﻿'
' Copyright (c) 2009-2015, MindFusion LLC - Bulgaria.
'

Imports MindFusion.Scheduling
Imports System.Xml.Linq


Partial Public Class _Default
    Inherits System.Web.UI.Page

    Private start As New DateTime(DateTime.Today.Year, DateTime.Today.Month, 1)
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        If Not IsPostBack Then
            Calendar1.[Date] = start
            Calendar1.EndDate = Calendar1.[Date].AddMonths(1)
            ' Populate the Schedule
            InitializeAppointments()
        End If
    End Sub

    ''' <summary>
    ''' Create a new CalendarEvent object from an Appointment object and update the undelrying XML data store.
    ''' </summary>
    ''' <param name="appointment">
    '''	the Appoinement object to create the CalendarEvent from.
    ''' </param>
    Private Sub CreateEvent(ByVal appointment As Appointment)
        appointment.Tag = New Guid(appointment.Id.Substring(3))
        Dim newEvent As CalendarEvent = EventToAppointmentConverter.CreateCalendarEvent(appointment)

        Dim events As XElement = XElement.Load(Server.MapPath(".") + "\\events.xml")
        events.Add(EventToAppointmentConverter.ConvertToXElement(newEvent))
        events.Save(Server.MapPath(".") + "\\events.xml")
    End Sub

    ''' <summary>
    ''' Delete a CalendarEvent object from the undelrying XML data store.
    ''' The CalendarEvent to delete is identified by the Appointment object.
    ''' </summary>
    ''' <param name="appointment">
    '''	the Appoinement object to identify the CalendarEvent with.
    ''' </param>
    Private Sub DeleteEvent(ByVal appointment As Appointment)
        Dim id As New Guid(appointment.Tag.ToString())

        Dim events As XElement = XElement.Load(Server.MapPath(".") + "\\events.xml")
		Dim deleted As XElement = events.Descendants("CalendarEvent").Where(Function(x) x.Descendants("Id").SingleOrDefault().Value = id.ToString()).SingleOrDefault()
		If deleted IsNot Nothing Then
			deleted.Remove()
		End If

        events.Save(Server.MapPath(".") + "\\events.xml")
    End Sub

    ''' <summary>
    ''' Delete a CalendarEvent object in the undelrying XML data store.
    ''' The CalendarEvent to modify is identified by the Appointment object.
    ''' </summary>
    ''' <param name="appointment">
    '''	the Appoinement object to identify the CalendarEvent with.
    ''' </param>
    Private Sub EditEvent(ByVal appointment As Appointment)
        Dim id As String = appointment.Tag.ToString()
        Dim editedEvent As New CalendarEvent()
        editedEvent.CopyFrom(EventToAppointmentConverter.CreateCalendarEvent(appointment))
        Dim events As XElement = XElement.Load(Server.MapPath(".") + "\\events.xml")
		Dim edited As XElement = events.Descendants("CalendarEvent").Where(Function(x) x.Descendants("Id").SingleOrDefault().Value = id).SingleOrDefault()
		If edited IsNot Nothing Then
			edited.ReplaceWith(EventToAppointmentConverter.ConvertToXElement(editedEvent))
		End If

        events.Save(Server.MapPath(".") + "\\events.xml")
    End Sub

    ''' <summary>
    ''' Synchronize the Schedule with the underlying XML data store.
    ''' </summary>
    Private Sub InitializeAppointments()
        Calendar1.Schedule.Clear()
        If Not System.IO.File.Exists(Server.MapPath(".") + "\\events.xml") Then
            RecreateFile()
        End If
        Dim query = From c In XElement.Load(Server.MapPath(".") + "\\events.xml").Elements()
        For Each evt In query
            Dim calEvt As CalendarEvent = EventToAppointmentConverter.ConvertFromXElement(evt)
            Dim app As Appointment = EventToAppointmentConverter.CreateAppointment(calEvt)
            Calendar1.Schedule.Items.Add(app)
        Next
    End Sub

    ''' <summary>
    ''' Recreate the XML data store.
    ''' </summary>
    Private Sub RecreateFile()
        Dim tempStore As New List(Of CalendarEvent)()

        Dim evt As New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Have Fun!", .Details = "Work hard, but sometimes we need to rest.", .StartTime = start + TimeSpan.FromDays(8) + TimeSpan.FromHours(10), .Duration = TimeSpan.FromHours(6)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Lunch with Laura", .Details = "Never to forget the appointment with the boss.", .StartTime = start + TimeSpan.FromHours(12), .Duration = TimeSpan.FromHours(3.5)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Get the children from the kindergarten!", .Details = "Yea!", .StartTime = start + TimeSpan.FromDays(7) + TimeSpan.FromHours(17), .Duration = TimeSpan.FromHours(0.45)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "William's birthday", .Details = "Party!", .StartTime = start + TimeSpan.FromDays(1), .Duration = TimeSpan.FromDays(1)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Visit the dentist", .Details = "This reminder is rather pointless, since my tooth is a reminder on its own", .StartTime = start + TimeSpan.FromDays(9) + TimeSpan.FromDays(2), .Duration = TimeSpan.FromDays(1)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Arrange the house payments!", .Details = "Yea!", .StartTime = start + TimeSpan.FromDays(2) + TimeSpan.FromHours(17.5), .Duration = TimeSpan.FromHours(3.5)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Meet George", .Details = "This is the seventh time I promise George. This time I should go.", .StartTime = start - TimeSpan.FromDays(-10) + TimeSpan.FromHours(8.5), .Duration = TimeSpan.FromHours(1)}
        tempStore.Add(evt)

        evt = New CalendarEvent() With {.Id = Guid.NewGuid(), .Name = "Prepare the dinner!", .Details = "Yea!", .StartTime = start + TimeSpan.FromDays(12) + TimeSpan.FromHours(18), .Duration = TimeSpan.FromHours(1)}
        tempStore.Add(evt)

        Dim list = New XElement("CalendarEvents", From item In tempStore Select EventToAppointmentConverter.ConvertToXElement(item))
        list.Save(Server.MapPath(".") + "\\events.xml")
    End Sub

    Protected Sub Calendar1_ItemCreated(ByVal sender As Object, ByVal e As MindFusion.Scheduling.ItemEventArgs) Handles Calendar1.ItemCreated
        CreateEvent(TryCast(e.Item, Appointment))
        InitializeAppointments()
    End Sub

    Protected Sub Calendar1_ItemDeleted(ByVal sender As Object, ByVal e As MindFusion.Scheduling.ItemEventArgs) Handles Calendar1.ItemDeleted
        DeleteEvent(TryCast(e.Item, Appointment))
        InitializeAppointments()
    End Sub

    Protected Sub Calendar1_ItemInplaceEdited(ByVal sender As Object, ByVal e As MindFusion.Scheduling.ItemEventArgs) Handles Calendar1.ItemInplaceEdited
        EditEvent(TryCast(e.Item, Appointment))
        InitializeAppointments()
    End Sub

    Protected Sub Calendar1_ItemModified(ByVal sender As Object, ByVal e As MindFusion.Scheduling.ItemModifiedEventArgs) Handles Calendar1.ItemModified
        EditEvent(TryCast(e.Item, Appointment))
        InitializeAppointments()
    End Sub
End Class
